@file:OptIn(ExperimentalMaterialApi::class)

package cz.uhk.dod.ui

import androidx.compose.animation.AnimatedVisibility
import androidx.compose.animation.core.LinearEasing
import androidx.compose.animation.core.tween
import androidx.compose.animation.fadeIn
import androidx.compose.animation.fadeOut
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.verticalScroll
import androidx.compose.material.*
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.ContactPage
import androidx.compose.material.icons.outlined.EventNote
import androidx.compose.material.icons.outlined.Info
import androidx.compose.material.icons.outlined.MoreVert
import androidx.compose.material.icons.twotone.*
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.vector.ImageVector
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.core.text.parseAsHtml
import androidx.navigation.NavHostController
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.currentBackStackEntryAsState
import cz.uhk.dod.R
import cz.uhk.dod.data.db.EventEntity
import cz.uhk.dod.data.db.FieldOfStudyEntity
import cz.uhk.dod.theme.FimIcons
import cz.uhk.dod.theme.spacing
import cz.uhk.dod.tools.dashWhenEmpty
import cz.uhk.dod.tools.replaceReturns
import cz.uhk.dod.tools.toAnnotatedString
import cz.uhk.dod.tools.toAppDate
import cz.uhk.dod.ui.base.Failure
import cz.uhk.dod.ui.base.Loading
import cz.uhk.dod.ui.pages.*
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import org.koin.androidx.compose.get

@Composable
fun AppContainer(
    controller: NavHostController,
    viewModel: DODViewModel = get(),
) {

    val scaffoldState = rememberBottomSheetScaffoldState(
        bottomSheetState = BottomSheetState(
            initialValue = BottomSheetValue.Collapsed,
        ),
        snackbarHostState = SnackbarHostState(),
    )

    val eventDetail = remember { mutableStateOf<EventEntity?>(null) }
    val fieldDetail = remember { mutableStateOf<FieldOfStudyEntity?>(null) }

    val scope = rememberCoroutineScope()

    val state = viewModel.launchState.collectAsState()

    val infoState = viewModel.infoState.collectAsState()
    val eventsState = viewModel.eventState.collectAsState()
    val videosState = viewModel.videoState.collectAsState()

    when (state.value) {
        is Failure -> {
            LaunchedEffect("LaunchStateSnack") {
                if (scaffoldState.snackbarHostState.showSnackbar(
                        message = "Data nebyla aktulizována",
                        actionLabel = "Zkusit znovu",
                        duration = SnackbarDuration.Short,
                    ) == SnackbarResult.ActionPerformed
                ) {
                    viewModel.fetchAll()
                }
            }
        }
        else -> {
            // Ignored for now
        }
    }

    BottomSheetScaffold(
        scaffoldState = scaffoldState,
        sheetPeekHeight = 0.dp,
        sheetShape = RoundedCornerShape(
            topStart = MaterialTheme.spacing.large,
            topEnd = MaterialTheme.spacing.large,
        ),
        sheetBackgroundColor = Color.Transparent,
        sheetContent = {
            if (eventDetail.value != null
                || fieldDetail.value != null
            ) {
                Box(
                    modifier = Modifier
                        .fillMaxWidth()
                        .fillMaxHeight(fraction = 0.8F)
                        .background(
                            color = MaterialTheme.colors.background,
                        ),
                ) {
                    eventDetail.value?.let { event ->
                        EventDetail(event)
                    }
                    fieldDetail.value?.let { field ->
                        FieldDetail(field)
                    }
                    IconButton(
                        modifier = Modifier
                            .align(Alignment.TopEnd)
                            .background(color = MaterialTheme.colors.background)
                            .padding(all = MaterialTheme.spacing.small),
                        onClick = {
                            scope.launch {
                                eventDetail.value = null
                                fieldDetail.value = null
                                scaffoldState.bottomSheetState.collapse()
                            }
                        },
                    ) {
                        Icon(
                            imageVector = Icons.TwoTone.Close,
                            contentDescription = null,
                        )
                    }
                }
            }
        },
        drawerElevation = 0.dp,
        sheetElevation = 0.dp,
    ) {
        Box {

            Scaffold(
                bottomBar = {
                    BottomNav(controller = controller)
                }
            ) {

                NavHost(
                    navController = controller,
                    startDestination = DestinationInfo,
                    modifier = Modifier.padding(bottom = it.calculateBottomPadding())
                ) {
                    composable(Routes.Info.route) {
                        InfoScreen(viewModel) { field ->
                            scope.launch {
                                fieldDetail.value = field
                                eventDetail.value = null
                                delay(200L)
                                scaffoldState.bottomSheetState.expand()
                            }
                        }
                    }
                    composable(Routes.Programme.route) {
                        EventScreen(viewModel) { event ->
                            scope.launch {
                                eventDetail.value = event
                                fieldDetail.value = null
                                delay(200L)
                                scaffoldState.bottomSheetState.expand()
                            }
                        }
                    }
                    composable(Routes.Contacts.route) {
                        ContactScreen()
                    }
                    composable(Routes.More.route) {
                        MoreScreen(viewModel)
                    }
                }
            }

            AnimatedVisibility(
                visible = state.value == Loading
                        || infoState.value == Loading
                        || eventsState.value == Loading
                        || videosState.value == Loading,
                enter = fadeIn(),
                exit = fadeOut(),
            ) {
                Box(
                    contentAlignment = Alignment.Center,
                    modifier = Modifier
                        .background(MaterialTheme.colors.background.copy(alpha = 0.75F))
                        .clickable { } // Disable click when loading
                        .fillMaxSize(),
                ) {
                    CircularProgressIndicator(
                        strokeWidth = 8.dp,
                        modifier = Modifier.size(80.dp)
                    )
                }
            }

            AnimatedVisibility(
                visible = scaffoldState.bottomSheetState.isCollapsed.not()
                        || scaffoldState.bottomSheetState.isAnimationRunning,
                enter = fadeIn(
                    animationSpec = tween(
                        durationMillis = 100,
                        easing = LinearEasing,
                    ),
                ),
                exit = fadeOut(
                    animationSpec = tween(
                        durationMillis = 100,
                        easing = LinearEasing,
                    ),
                )
            ) {
                Box(
                    modifier = Modifier
                        .fillMaxSize()
                        .clickable {
                            scope.launch {
                                eventDetail.value = null
                                fieldDetail.value = null
                                scaffoldState.bottomSheetState.collapse()
                            }
                        }
                        .background(
                            color = Color.Black.copy(alpha = 0.5F),
                        )
                )
            }
        }
    }
}

@Composable
fun EmptyView(
    onClick: () -> Unit,
) {
    Column(
        modifier = Modifier.fillMaxSize().padding(MaterialTheme.spacing.double),
        verticalArrangement = Arrangement.Center,
        horizontalAlignment = Alignment.CenterHorizontally,
    ) {
        Text(
            text = "Aktuálně nejsou k dispozici data k zobrazení",
            style = MaterialTheme.typography.h4,
        )
        Spacer(Modifier.height(MaterialTheme.spacing.medium))
        Text(
            text = "Zkuste data aktualizovat ikonou 👇🏼",
            style = MaterialTheme.typography.body1,
        )
        Spacer(Modifier.height(MaterialTheme.spacing.double))
        IconButton(
            onClick = onClick,
        ) {
            Icon(
                imageVector = Icons.TwoTone.Sync,
                contentDescription = null,
                modifier = Modifier.size(60.dp),
            )
        }
    }
}

@Composable
fun FieldDetail(field: FieldOfStudyEntity) {
    Column(
        modifier = Modifier
            .fillMaxSize()
            .verticalScroll(rememberScrollState())
            .padding(all = MaterialTheme.spacing.medium),
    ) {

        Spacer(Modifier.height(MaterialTheme.spacing.large))
        Text(
            text = field.name,
            style = MaterialTheme.typography.h3,
        )

        Spacer(Modifier.height(MaterialTheme.spacing.medium))
        IconText(
            icon = Icons.TwoTone.CheckCircle,
            size = 24.dp,
            space = MaterialTheme.spacing.medium,
            subtext = {
                Text(
                    text = field.accreditation.toAppDate,
                    style = MaterialTheme.typography.body1,
                    color = MaterialTheme.colors.onBackground,
                )
            }
        ) {
            Text(
                text = "Akreditace do",
                style = MaterialTheme.typography.h5,
                color = MaterialTheme.colors.onBackground,
            )
        }

        Spacer(Modifier.height(MaterialTheme.spacing.medium))
        IconText(
            icon = Icons.TwoTone.HowToReg,
            size = 24.dp,
            space = MaterialTheme.spacing.medium,
            subtext = {
                Text(
                    text = field.candidateProfile.dashWhenEmpty.replaceReturns,
                    style = MaterialTheme.typography.body1,
                    color = MaterialTheme.colors.onBackground,
                )
            }
        ) {
            Text(
                text = "Profil kandidáta",
                style = MaterialTheme.typography.h5,
                color = MaterialTheme.colors.onBackground,
            )
        }

        Spacer(Modifier.height(MaterialTheme.spacing.medium))
        IconText(
            icon = Icons.TwoTone.School,
            size = 24.dp,
            space = MaterialTheme.spacing.medium,
            subtext = {
                Text(
                    text = field.graduateProfile.dashWhenEmpty.replaceReturns,
                    style = MaterialTheme.typography.body1,
                    color = MaterialTheme.colors.onBackground,
                )
            }
        ) {
            Text(
                text = "Profil absolventa",
                style = MaterialTheme.typography.h5,
                color = MaterialTheme.colors.onBackground,
            )
        }

        Spacer(Modifier.height(MaterialTheme.spacing.medium))
        IconText(
            icon = Icons.TwoTone.AdminPanelSettings,
            size = 24.dp,
            space = MaterialTheme.spacing.medium,
            subtext = {
                Text(
                    text = field.admissionProcedure.dashWhenEmpty.replaceReturns,
                    style = MaterialTheme.typography.body1,
                    color = MaterialTheme.colors.onBackground,
                )
            }
        ) {
            Text(
                text = "Přijímací řízení",
                style = MaterialTheme.typography.h5,
                color = MaterialTheme.colors.onBackground,
            )
        }

        Spacer(Modifier.height(MaterialTheme.spacing.medium))
        IconText(
            icon = Icons.TwoTone.FollowTheSigns,
            size = 24.dp,
            space = MaterialTheme.spacing.medium,
            subtext = {
                Text(
                    text = field.furtherStudyPossibility.dashWhenEmpty.replaceReturns,
                    style = MaterialTheme.typography.body1,
                    color = MaterialTheme.colors.onBackground,
                )
            }
        ) {
            Text(
                text = "Možnosti dalšího vzdělávání",
                style = MaterialTheme.typography.h5,
                color = MaterialTheme.colors.onBackground,
            )
        }

        Spacer(Modifier.height(MaterialTheme.spacing.medium))
        IconText(
            icon = Icons.TwoTone.Notes,
            size = 24.dp,
            space = MaterialTheme.spacing.medium,
            subtext = {
                Text(
                    text = field.description.dashWhenEmpty.replaceReturns,
                    style = MaterialTheme.typography.body1,
                    color = MaterialTheme.colors.onBackground,
                )
            }
        ) {
            Text(
                text = "Popis",
                style = MaterialTheme.typography.h5,
                color = MaterialTheme.colors.onBackground,
            )
        }
    }
}

@Composable
fun EventDetail(event: EventEntity) {
    Column(
        modifier = Modifier
            .fillMaxSize()
            .verticalScroll(rememberScrollState())
            .padding(all = MaterialTheme.spacing.medium),
    ) {

        Spacer(Modifier.height(MaterialTheme.spacing.large))
        Text(
            text = event.name,
            style = MaterialTheme.typography.h3,
        )
        Spacer(Modifier.height(MaterialTheme.spacing.medium))

        IconText(
            icon = Icons.TwoTone.Alarm,
            size = 24.dp,
            space = MaterialTheme.spacing.medium,
        ) {
            val (from, to) = event.times
            Text(
                text = "${from.timeString}-${to.timeString}",
                style = MaterialTheme.typography.h5,
                color = MaterialTheme.colors.onBackground,
            )
        }

        Spacer(Modifier.height(MaterialTheme.spacing.tiny))
        IconText(
            icon = Icons.TwoTone.Place,
            size = 24.dp,
            space = MaterialTheme.spacing.medium,
        ) {
            Text(
                text = event.location,
                style = MaterialTheme.typography.h5,
                color = MaterialTheme.colors.onBackground,
            )
        }
        if (event.repetitionTimeInMinutes != null) {
            Spacer(Modifier.height(MaterialTheme.spacing.tiny))
            IconText(
                icon = Icons.TwoTone.Timer,
                size = 24.dp,
                space = MaterialTheme.spacing.medium,
            ) {
                Text(
                    text = "každých ${event.repetitionTimeInMinutes} minut",
                    style = MaterialTheme.typography.h5,
                    color = MaterialTheme.colors.onBackground,
                )
            }
        }

        Spacer(Modifier.height(MaterialTheme.spacing.large))

        Text(
            text = event.description.parseAsHtml().toAnnotatedString(),
            style = MaterialTheme.typography.body1,
        )
    }
}

@Composable
fun BottomNav(
    controller: NavHostController,
) {
    val items = Routes.values()

    BottomNavigation(
        backgroundColor = MaterialTheme.colors.background,
        contentColor = MaterialTheme.colors.secondary,
    ) {

        val navBackStackEntry by controller.currentBackStackEntryAsState()
        val currentRoute = navBackStackEntry?.destination?.route

        items.forEach { item ->

            val title = stringResource(item.label)

            BottomNavigationItem(
                icon = {
                    Icon(
                        imageVector = item.icon,
                        contentDescription = title,
                    )
                },
                label = {
                    Text(
                        text = title,
                        fontSize = 10.sp,
                        style = MaterialTheme.typography.body1,
                    )
                },
                modifier = Modifier,
                selectedContentColor = MaterialTheme.colors.primary,
                unselectedContentColor = MaterialTheme.colors.primaryVariant,
                alwaysShowLabel = true,
                selected = currentRoute == item.route,
                onClick = {
                    controller.navigate(item.route) {
                        controller.graph.startDestinationRoute?.let { route ->
                            popUpTo(route) {
                                saveState = true
                            }
                        }
                        launchSingleTop = true
                        restoreState = true
                    }
                }
            )
        }
    }
}

private const val DestinationInfo = "info"
private const val DestinationProgramme = "programme"
private const val DestinationContacts = "contacts"
private const val DestinationMore = "more"

sealed class Routes(
    val icon: ImageVector,
    val route: String,
    val label: Int,
) {
    object Info : Routes(FimIcons.Info, DestinationInfo, R.string.info)
    object Programme : Routes(FimIcons.EventNote, DestinationProgramme, R.string.programme)
    object Contacts : Routes(FimIcons.ContactPage, DestinationContacts, R.string.contacts)
    object More : Routes(FimIcons.MoreVert, DestinationMore, R.string.more)

    companion object {
        fun values() = listOf(
            Info, Programme, Contacts, More
        )
    }
}