package cz.uhk.dod.ui.base

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.material.*
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import cz.uhk.dod.tools.isNetworkConnection

/**
 * Base screen has similar function as a BaseFragment. It contains content of a child
 * and at the same time shows progress bar and reacts to failures in viewModel.
 *
 * @param state ComposeViewModel state for handling launch states.
 * @param showFullscreenLoading Used for disabling main progress bar if you have your own.
 * @param content Content of your child screen.
 *
 * @author Dusan Salay
 */
@Composable
fun BaseScreen(
    state: LaunchState,
    showFullscreenLoading: Boolean = true,
    onDismiss: () -> Unit,
    content: @Composable () -> Unit,
) {
    // Child Content first
    content()
    // Reaction to the state of viewModels
    when (state) {

        is Loading, // If you have custom loading indicator, then [showFullscreenLoading=false]
        -> if (showFullscreenLoading) LoadingIndicator()

        is Failure,
        -> ErrorDialog(
            failure = state,
            onConfirm = { onDismiss() },
            onDismiss = onDismiss,
        )

        else -> Unit // Ignored
    }
}

@Composable
fun LoadingIndicator() {
    Box(
        modifier = Modifier
            .fillMaxHeight()
            .fillMaxWidth()
            .background(MaterialTheme.colors.background)

    ) {
        CircularProgressIndicator(
            modifier = Modifier
                .align(Alignment.Center),
            color = MaterialTheme.colors.onBackground,
        )
    }
}

@Composable
fun ErrorDialog(
    failure: Failure,
    onConfirm: () -> Unit,
    onDismiss: () -> Unit,
) {
    // When new screens are added ad here custom exception handlers if necessary

    val (title, message, positive) =
        when {
            failure.exception.isNetworkConnection
            -> ErrorData(
                message = "Network issues",
                positiveButton = "Ok",
            )
            else
            -> ErrorData(
                message = failure.exception.message.toString(),
                positiveButton = "Ok",
            )
        }

    AlertDialog(
        title = {
            if (title != null)
                Text(text = title.orEmpty())
        },
        text = {
            if (message != null)
                Text(text = message.orEmpty())
        },
        confirmButton = {
            if (positive != null)
                Button(
                    onClick = onConfirm,
                ) {
                    Text("Ok")
                }
        },
        onDismissRequest = onDismiss,
    )
}

data class ErrorData(
    val title: String? = null,
    val message: String? = null,
    val positiveButton: String? = null,
)
