package cz.uhk.dod.ui.base

import androidx.lifecycle.ViewModel
import kotlinx.coroutines.*
import kotlinx.coroutines.flow.MutableStateFlow

sealed class LaunchState
object None : LaunchState()
object Loading : LaunchState()
class Success(val any: Any?) : LaunchState()
class Failure(val exception: Exception) : LaunchState()

/**
 * View model for @Composable functions
 *
 * @sample ViewModel: class MyViewModel : ComposeViewModel() { ... }
 * @sample Unit: @Composable fun MyView(myViewModel: MyViewModel = get()) { ... } // Your viewmodel injected by Koin
 */
abstract class ComposeViewModel : ViewModel() {

    //Children of a supervisor job can fail independently of each other.
    private val job = SupervisorJob()
    private val scope = CoroutineScope(Dispatchers.Default + job)

    val launchState = MutableStateFlow<LaunchState>(None)

    /**
     * @param block     Your block of application instructions.
     * @param state     State where are the lifecycle of launch is emitted. You can insert null
     *                  if you don't care about the lifecycle.
     * @param onError   Optional onError method that is called extra when error appears.
     */
    protected fun <Result> launch(
        onError: ((Exception) -> Unit)? = null,
        state: MutableStateFlow<LaunchState>? = this.launchState,
        block: (suspend CoroutineScope.() -> Result),
    ) = scope.launch(
        CoroutineExceptionHandler { _, error ->
            // Handle possible exception
            when (error) {
                is Exception -> {
                    onError?.invoke(error)
                    state?.tryEmit(Failure(error))
                }
                else -> {
                    // Optional
                }
            }
        }
    ) {

        // Show loading via emitted in state
        state?.emit(Loading)

        // Process operation, crate Success and emit if state available
        block()
            .let(::Success)
            .also { state?.emit(it) }
    }

    protected fun <Item> MutableStateFlow<Item>.emit(emitter: Item.() -> Item) {
        scope.launch { this@emit.emit(value.emitter()) }
    }

    //zavolat pokud by neco trvalo moc dlouho a chtel bych vycistit state
    fun clearState(state: MutableStateFlow<LaunchState> = launchState) {
        launchState.emit { None }
    }

    override fun onCleared() {
        super.onCleared()
        job.cancel()
    }
}
