@file:OptIn(ExperimentalMaterial3Api::class, ExperimentalFoundationApi::class,
    ExperimentalMaterialApi::class)

package cz.uhk.dod.ui.pages

import androidx.compose.animation.rememberSplineBasedDecay
import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.ExperimentalMaterialApi
import androidx.compose.material.MaterialTheme
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.twotone.Place
import androidx.compose.material.icons.twotone.Sync
import androidx.compose.material.icons.twotone.Timer
import androidx.compose.material3.*
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.input.nestedscroll.nestedScroll
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import cz.uhk.dod.data.api.response.UniversityEventType
import cz.uhk.dod.data.db.EventEntity
import cz.uhk.dod.theme.elevation
import cz.uhk.dod.theme.spacing
import cz.uhk.dod.ui.DODViewModel
import cz.uhk.dod.ui.EmptyView
import java.util.*

@Composable
fun EventScreen(
    viewModel: DODViewModel,
    openEvent: (EventEntity) -> Unit,
) {

    val decayAnimationSpec = rememberSplineBasedDecay<Float>()
    val scrollBehavior = TopAppBarDefaults.exitUntilCollapsedScrollBehavior(
        decayAnimationSpec = decayAnimationSpec,
        state = rememberTopAppBarState()
    )

    val events = viewModel.events.collectAsState(
        initial = UniversityEventType.PRESENTATION to emptyList(),
    )

    Scaffold(
        modifier = Modifier.nestedScroll(scrollBehavior.nestedScrollConnection),
        topBar = {
            LargeTopAppBar(
                title = {
                    Text(
                        text = events.value.first.label,
                    )
                },
                colors = TopAppBarDefaults.largeTopAppBarColors(
                    containerColor = MaterialTheme.colors.background,
                    titleContentColor = MaterialTheme.colors.onBackground,
                ),
                actions = {
                    IconButton(
                        onClick = {
                            viewModel.fetchEvents()
                        }
                    ) {
                        Icon(
                            imageVector = Icons.TwoTone.Sync,
                            contentDescription = "Refresh",
                        )
                    }
                },
                scrollBehavior = scrollBehavior
            )
        },
        containerColor = MaterialTheme.colors.background,
        floatingActionButton = {
            Chips {
                viewModel.selectEventType(it)
            }
        },
        content = { innerPadding ->
            LazyColumn(
                contentPadding = PaddingValues(
                    top = innerPadding.calculateTopPadding(),
                    bottom = 80.dp,
                ),
            ) {
                if (events.value.second.isNotEmpty()) {
                    items(events.value.second) { item ->

                        Row(
                            verticalAlignment = Alignment.CenterVertically,
                            modifier = Modifier
                                .fillMaxWidth()
                                .clickable {
                                    openEvent(item)
                                }
                                .animateItemPlacement(),
                        ) {

                            Column(
                                horizontalAlignment = Alignment.CenterHorizontally,
                                modifier = Modifier
                                    .width(80.dp)
                                    .padding(all = MaterialTheme.spacing.medium)
                                    .wrapContentHeight(align = Alignment.CenterVertically)
                            ) {
                                val (from, to) = item.times
                                Text(
                                    text = from.timeString,
                                    style = MaterialTheme.typography.h5,
                                    color = MaterialTheme.colors.onBackground,
                                )
                                Text(
                                    text = to.timeString,
                                    style = MaterialTheme.typography.h5,
                                    color = MaterialTheme.colors.onBackground,
                                )
                            }

                            Box(
                                modifier = Modifier
                                    .height(40.dp)
                                    .width(1.dp)
                                    .background(color = MaterialTheme.colors.primary)
                            )

                            Column(
                                modifier = Modifier
                                    .weight(1F)
                                    .padding(all = MaterialTheme.spacing.medium)
                            ) {
                                Text(
                                    text = item.name,
                                    style = MaterialTheme.typography.h3,
                                    color = MaterialTheme.colors.onBackground,
                                )
                                Spacer(Modifier.height(MaterialTheme.spacing.small))
                                IconText(
                                    icon = Icons.TwoTone.Place,
                                    size = 12.dp,
                                    space = MaterialTheme.spacing.small,
                                ) {
                                    Text(
                                        text = item.location,
                                        style = MaterialTheme.typography.h5,
                                        color = MaterialTheme.colors.onBackground,
                                    )
                                }
                                if (item.repetitionTimeInMinutes != null) {
                                    Spacer(Modifier.height(MaterialTheme.spacing.tiny))
                                    IconText(
                                        icon = Icons.TwoTone.Timer,
                                        size = 12.dp,
                                        space = MaterialTheme.spacing.small,
                                    ) {
                                        Text(
                                            text = "každých ${item.repetitionTimeInMinutes} minut",
                                            style = MaterialTheme.typography.h5,
                                            color = MaterialTheme.colors.onBackground,
                                        )
                                    }
                                }
                            }
                        }
                    }
                } else item { EmptyView(viewModel::fetchEvents) }
            }
        }
    )
}

val Calendar.timeString
    get() = "${
        get(Calendar.HOUR_OF_DAY).toString().padStart(2, '0')
    }:${
        get(Calendar.MINUTE).toString().padStart(2, '0')
    }"

@Composable
fun Chips(
    onSelected: (Int) -> Unit,
) {
    Box(
        modifier = Modifier
            .padding(start = 32.dp)
            .sizeIn(maxHeight = 70.dp),
    ) {
        Card(
            colors = CardDefaults.cardColors(
                containerColor = MaterialTheme.colors.background,
            ),
            elevation = CardDefaults.cardElevation(
                defaultElevation = MaterialTheme.elevation.medium,
            ),
            shape = RoundedCornerShape(8.dp),
        ) {
            Row(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(
                        all = MaterialTheme.spacing.small,
                    )
            ) {

                val chipGroup = remember { mutableStateOf(0) }

                UniversityEventType.values().forEachIndexed { index, type ->

                    if (index > 0) Spacer(Modifier.width(MaterialTheme.spacing.small))

                    Chip(
                        name = type.label,
                        index = index,
                        chipGroup = chipGroup,
                        modifier = Modifier
                            .weight(1F),
                        onSelected = onSelected,
                    )
                }
            }
        }
    }
}

@Composable
fun Chip(
    name: String,
    index: Int,
    chipGroup: MutableState<Int>,
    modifier: Modifier = Modifier,
    onSelected: (Int) -> Unit,
) {
    val isSelected = chipGroup.value == index
    Card(
        shape = MaterialTheme.shapes.medium,
        modifier = modifier
            .fillMaxHeight()
            .clickable {
                chipGroup.value = index
                onSelected(index)
            },
        colors = CardDefaults.cardColors(
            containerColor = if (isSelected.not()) MaterialTheme.colors.secondary
            else MaterialTheme.colors.primary
        )
    ) {
        Row(
            horizontalArrangement = Arrangement.Center,
            verticalAlignment = Alignment.CenterVertically,
            modifier = Modifier.fillMaxHeight(),
        ) {
            Text(
                text = name,
                style = MaterialTheme.typography.body1,
                color = if (isSelected.not()) MaterialTheme.colors.onSecondary
                else MaterialTheme.colors.onPrimary,
                textAlign = TextAlign.Center,
                modifier = Modifier
                    .padding(8.dp)
                    .fillMaxWidth(),
            )
        }
    }
}