@file:OptIn(ExperimentalMaterial3Api::class)

package cz.uhk.dod.ui.pages

import androidx.compose.animation.*
import androidx.compose.animation.core.FastOutLinearInEasing
import androidx.compose.animation.core.animateFloatAsState
import androidx.compose.animation.core.tween
import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.material.MaterialTheme
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.ChevronRight
import androidx.compose.material.icons.twotone.*
import androidx.compose.material3.*
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.rotate
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.vector.ImageVector
import androidx.compose.ui.input.nestedscroll.nestedScroll
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.dp
import cz.uhk.dod.R
import cz.uhk.dod.data.db.FieldOfStudyEntity
import cz.uhk.dod.theme.FimIcons
import cz.uhk.dod.theme.spacing
import cz.uhk.dod.ui.DODViewModel
import cz.uhk.dod.ui.EmptyView
import cz.uhk.dod.ui.Routes

@Composable
fun InfoScreen(
    viewModel: DODViewModel,
    openFieldOfStudy: (FieldOfStudyEntity) -> Unit,
) {

    val decayAnimationSpec = rememberSplineBasedDecay<Float>()
    val scrollBehavior = TopAppBarDefaults.exitUntilCollapsedScrollBehavior(
        decayAnimationSpec = decayAnimationSpec,
        state = rememberTopAppBarState()
    )

    val aboutFaculty = remember { mutableStateOf(false) }
    val programmesOverview = remember { mutableStateOf(false) }

    val fieldsOfStudy = viewModel.fieldsOfStudy.collectAsState(emptyMap())

    Box(
        modifier = Modifier
            .background(MaterialTheme.colors.background),
    ) {

        Image(
            painter = painterResource(R.drawable.ic_fim),
            contentDescription = null,
            modifier = Modifier
                .align(Alignment.BottomEnd),
        )

        Scaffold(
            modifier = Modifier.nestedScroll(scrollBehavior.nestedScrollConnection),
            topBar = {
                LargeTopAppBar(
                    title = {
                        Text(
                            text = stringResource(Routes.Info.label)
                        )
                    },
                    colors = TopAppBarDefaults.largeTopAppBarColors(
                        containerColor = MaterialTheme.colors.background,
                        titleContentColor = MaterialTheme.colors.onBackground,
                    ),
                    actions = {
                        IconButton(
                            onClick = {
                                viewModel.fetchInfo()
                            }
                        ) {
                            Icon(
                                imageVector = Icons.TwoTone.Sync,
                                contentDescription = "Refresh",
                            )
                        }
                    },
                    scrollBehavior = scrollBehavior
                )
            },
            containerColor = Color.Transparent,//MaterialTheme.colors.background,
            content = { innerPadding ->

                LazyColumn(
                    contentPadding = innerPadding,
                    verticalArrangement = Arrangement.spacedBy(MaterialTheme.spacing.large)
                ) {

                    item {

                        CollapsibleSection(
                            label = stringResource(R.string.about_faculty),
                            icon = Icons.TwoTone.Info,
                            expanded = aboutFaculty,
                        ) {
                            Text(
                                text = "Fakulta informatiky a managementu Univerzity Hradec Králové" +
                                        " je jednou ze čtyř součástí Univerzity Hradec Králové. " +
                                        "V roce 2000 se transformovala z původní Fakulty řízení a " +
                                        "informační technologie působící od roku 1993.",
                                modifier = Modifier.padding(all = MaterialTheme.spacing.medium),
                                style = MaterialTheme.typography.body1,
                                color = MaterialTheme.colors.onBackground,
                            )
                        }
                    }

                    item {

                        CollapsibleSection(
                            label = stringResource(R.string.programmes_overview),
                            icon = Icons.TwoTone.ViewAgenda,
                            expanded = programmesOverview,
                        ) {
                            if (fieldsOfStudy.value.keys.isNotEmpty()) {
                                Column(
                                    modifier = Modifier
                                        .padding(horizontal = MaterialTheme.spacing.small)
                                ) {

                                    fieldsOfStudy.value.keys.forEach { key ->

                                        val subCategory = remember { mutableStateOf(false) }

                                        CollapsibleSection(
                                            label = key.label,
                                            labelStyle = MaterialTheme.typography.h4,
                                            expanded = subCategory,
                                        ) {
                                            Column {
                                                fieldsOfStudy.value[key]?.forEach { field ->
                                                    Text(
                                                        text = field.name,
                                                        modifier = Modifier
                                                            .clickable {
                                                                openFieldOfStudy(field)
                                                            }
                                                            .fillMaxWidth()
                                                            .padding(all = MaterialTheme.spacing.medium),
                                                        color = MaterialTheme.colors.onBackground,
                                                    )
                                                }
                                            }
                                        }
                                    }
                                }
                            } else EmptyView(viewModel::fetchInfo)
                        }
                    }
                }
            }
        )
    }
}

@Composable
fun IconText(
    icon: ImageVector,
    size: Dp? = null,
    space: Dp = MaterialTheme.spacing.medium,
    modifier: Modifier = Modifier,
    tint: Color = MaterialTheme.colors.primary,
    subtext: (@Composable () -> Unit)? = null,
    text: @Composable () -> Unit,
) {
    Row(
        verticalAlignment = if (subtext != null) Alignment.Top
        else Alignment.CenterVertically,
        modifier = modifier,
    ) {
        Icon(
            imageVector = icon,
            contentDescription = "",
            tint = tint,
            modifier = Modifier
                .size(size = size ?: 24.dp)
        )
        Spacer(Modifier.width(space))
        Column {
            text()
            if (subtext != null) subtext()
        }
    }
}

@Composable
fun CollapsibleSection(
    label: String,
    labelStyle: TextStyle = MaterialTheme.typography.h3,
    icon: ImageVector? = null,
    expanded: MutableState<Boolean>,
    content: @Composable () -> Unit,
) {

    //for animation of chevron icon
    val angle = animateFloatAsState(
        targetValue = if (expanded.value) -90F else 90F,
        animationSpec = tween(
            durationMillis = 250,
            easing = FastOutLinearInEasing,
        )
    )

    Row(
        verticalAlignment = Alignment.CenterVertically,
        modifier = Modifier
            .clickable {
                expanded.value = expanded.value.not()
            }
            .padding(all = MaterialTheme.spacing.medium)
    ) {
        if (icon != null)
            IconText(
                icon = icon,
                modifier = Modifier.weight(1F)
            ) {
                Text(
                    text = label,
                    style = labelStyle,
                    color = MaterialTheme.colors.primaryVariant,
                )
            }
        else
            Text(
                text = label,
                style = labelStyle,
                color = MaterialTheme.colors.primaryVariant,
                modifier = Modifier.weight(1F)
            )
        Icon(
            imageVector = FimIcons.ChevronRight,
            contentDescription = "Indicator",
            tint = MaterialTheme.colors.primaryVariant,
            modifier = Modifier.rotate(angle.value)
        )
    }

    AnimatedVisibility(
        visible = expanded.value,
        enter = expandVertically() + fadeIn(),
        exit = shrinkVertically() + fadeOut(),
    ) {
        content()
    }
}